<?php

namespace App\Http\Controllers;

use App\DataTables\Approval\ApprovalGroupDataTable;
use App\Http\Requests\CreateApprovalGroupRequest;
use App\Http\Requests\UpdateApprovalGroupRequest;
use App\Http\Resources\ApprovalGroupCollection;
use App\Http\Resources\ApprovalGroupResource;
use App\Models\ApprovalGroup;
use App\Models\ApprovalGroupRole;
use App\Repositories\ApprovalGroupRepository;
use Illuminate\Database\Eloquent\ModelNotFoundException;
use Illuminate\Http\RedirectResponse;
use Illuminate\Support\Facades\DB;
use Illuminate\View\View;
//use RealRashid\SweetAlert\Facades\Alert;
//use Spatie\Permission\Models\Role;

class ApprovalGroupController extends Controller
{
    /** @var  ApprovalGroupRepository */
    private ApprovalGroupRepository $approvalGroupRepository;

    public function __construct(ApprovalGroupRepository $approvalGroupRepo)
    {
        $this->approvalGroupRepository = $approvalGroupRepo;
    }

    /**
     * Display a listing of the ApprovalGroup.
     *
     */
    public function index()
    {
        $approvalGroups = ApprovalGroup::all();
        return new ApprovalGroupCollection($approvalGroups);
    }

    /**
     * Show the form for creating a new ApprovalGroup.
     *
     * @return View
     */
    public function create(): View
    {
        return view('approval_groups.create');
    }

    /**
     * Store a newly created ApprovalGroup in storage.
     *
     * @param CreateApprovalGroupRequest $request
     *
     */
    public function store(CreateApprovalGroupRequest $request)
    {
        /*{
            "_token": "u8A2osUUZKl4AxO24xq2aaeUQU5MDYKpTLLge60J",
            "name": "group",
            "rank": "1",
            "role_id": [
                    "4",
                    "5"
                ],
            "description": "description for group"
        }*/

        $input = $request->all();

        $approvalGroup = new ApprovalGroup();
        DB::beginTransaction();
        try {

            //$approvalGroup = $this->approvalGroupRepository->create($input);
            $approvalGroup = ApprovalGroup::create([
                'name' => $input['name'],
                'rank' => $input['rank'],
                'updated_by' => $input['updated_by'],
                'created_by' => $input['created_by'],
                'description' => $input['description'],
            ]);

            foreach (json_decode($input['role_id']) as $roleID) {
               // $role = Role::findOrFail($roleID);
                //$approvalGroup->roles()->attach($role);
                ApprovalGroupRole::create(['approval_group_id' => $approvalGroup->id, 'role_id' => $roleID]);
            }

        } catch (ModelNotFoundException $e) {
            DB::rollback();
        }
        DB::commit();


        return new ApprovalGroupResource($approvalGroup);
    }

    /**
     * Display the specified ApprovalGroup.
     *
     * @param  int $id
     *
     * @return Response
     */
    public function show($id)
    {
        $approvalGroup = $this->approvalGroupRepository->findWithoutFail($id);

        if (empty($approvalGroup)) {
            Flash::error('Approval Group not found');

            return redirect(route('approvalGroups.index'));
        }

        return view('approval_groups.show')->with('approvalGroup', $approvalGroup);
    }

    /**
     * Show the form for editing the specified ApprovalGroup.
     *
     * @param  int $id
     *
     * @return Response
     */
    public function edit($id)
    {
        $approvalGroup = $this->approvalGroupRepository->findWithoutFail($id);

        if (empty($approvalGroup)) {
            Flash::error('Approval Group not found');

            return redirect(route('approvalGroups.index'));
        }

        return view('approval_groups.edit')->with('approvalGroup', $approvalGroup);
    }

    /**
     * Update the specified ApprovalGroup in storage.
     *
     * @param  int $id
     * @param UpdateApprovalGroupRequest $request
     *
     * @return Response
     */
    public function update($id, UpdateApprovalGroupRequest $request)
    {
        $approvalGroup = $this->approvalGroupRepository->findWithoutFail($id);

        if (empty($approvalGroup)) {
            Flash::error('Approval Group not found');

            return redirect(route('approvalGroups.index'));
        }
        //begin transaction
        DB::beginTransaction();
        try {

            $approvalGroup = $this->approvalGroupRepository->update($request->all(), $id);

        } catch (ModelNotFoundException $e) {
            DB::rollback();
            //dd(get_class_methods($e)); // lists all available methods for exception object

        }
        DB::commit();


        Flash::success('Approval Group updated successfully.');

        return redirect(route('approvalGroups.index'));
    }

    /**
     * Remove the specified ApprovalGroup from storage.
     *
     * @param  int $id
     *
     * @return Response
     */
    public function destroy($id)
    {
        $approvalGroup = $this->approvalGroupRepository->findWithoutFail($id);

        if (empty($approvalGroup)) {
            Flash::error('Approval Group not found');

            return redirect(route('approvalGroups.index'));
        }

        $this->approvalGroupRepository->delete($id);

        Flash::success('Approval Group deleted successfully.');

        return redirect(route('approvalGroups.index'));
    }

    /**
     * show  approval groups in selector2  .
     *
     * @return Response
     */
    public function selector()
    {
        $approvalGroups = ApprovalGroup::pluck('name', 'id');
        foreach ($approvalGroups as $key => $value) {

            $approvalGroupSelector[] = ['id' => $key, 'text' => $value];
        }
        return response()->json(['results' => $approvalGroupSelector], 200);
    }
}
