<?php

namespace App\Http\Controllers;

use App\Http\Requests\CreateApprovalHistoryRequest;
use App\Http\Requests\UpdateApprovalHistoryRequest;
use App\Http\Resources\ApprovalHistoryCollection;
use App\Repositories\ApprovalHistoryRepository;

class ApprovalHistoryController extends Controller
{
    /** @var  ApprovalHistoryRepository */
    private $approvalHistoryRepository;

    public function __construct(ApprovalHistoryRepository $approvalHistoryRepo)
    {
       // $this->middleware('auth');
        $this->approvalHistoryRepository = $approvalHistoryRepo;
    }


    public function index()
    {
        $histories = $this->approvalHistoryRepository->all();
        return new ApprovalHistoryCollection($histories);
    }

    /**
     * Show the form for creating a new ApprovalHistory.
     *
     * @return Response
     */
    public function create()
    {
        return view('approval_histories.create');
    }

    /**
     * Store a newly created ApprovalHistory in storage.
     *
     * @param CreateApprovalHistoryRequest $request
     *
     * @return Response
     */
    public function store(CreateApprovalHistoryRequest $request)
    {
        $input = $request->all();

        $approvalHistory = $this->approvalHistoryRepository->create($input);

        Flash::success('Approval History saved successfully.');

        return redirect(route('approvalHistories.index'));
    }

    /**
     * Display the specified ApprovalHistory.
     *
     * @param  int $id
     *
     * @return Response
     */
    public function show($id)
    {
        $approvalHistory = $this->approvalHistoryRepository->findWithoutFail($id);

        if (empty($approvalHistory)) {
            Flash::error('Approval History not found');

            return redirect(route('approvalHistories.index'));
        }

        return view('approval_histories.show')->with('approvalHistory', $approvalHistory);
    }

    /**
     * Show the form for editing the specified ApprovalHistory.
     *
     * @param  int $id
     *
     * @return Response
     */
    public function edit($id)
    {
        $approvalHistory = $this->approvalHistoryRepository->findWithoutFail($id);

        if (empty($approvalHistory)) {
            Flash::error('Approval History not found');

            return redirect(route('approvalHistories.index'));
        }

        return view('approval_histories.edit')->with('approvalHistory', $approvalHistory);
    }

    /**
     * Update the specified ApprovalHistory in storage.
     *
     * @param  int              $id
     * @param UpdateApprovalHistoryRequest $request
     *
     * @return Response
     */
    public function update($id, UpdateApprovalHistoryRequest $request)
    {
        $approvalHistory = $this->approvalHistoryRepository->findWithoutFail($id);

        if (empty($approvalHistory)) {
            Flash::error('Approval History not found');

            return redirect(route('approvalHistories.index'));
        }

        $approvalHistory = $this->approvalHistoryRepository->update($request->all(), $id);

        Flash::success('Approval History updated successfully.');

        return redirect(route('approvalHistories.index'));
    }

    /**
     * Remove the specified ApprovalHistory from storage.
     *
     * @param  int $id
     *
     * @return Response
     */
    public function destroy($id)
    {
        $approvalHistory = $this->approvalHistoryRepository->findWithoutFail($id);

        if (empty($approvalHistory)) {
            Flash::error('Approval History not found');

            return redirect(route('approvalHistories.index'));
        }

        $this->approvalHistoryRepository->delete($id);

        Flash::success('Approval History deleted successfully.');

        return redirect(route('approvalHistories.index'));
    }
}
