<?php

namespace App\Http\Controllers;


use App\Http\Requests\Requisition;
use App\Http\Requests\CreateApprovalWorkFlowRequest;
use App\Http\Requests\UpdateApprovalWorkFlowRequest;
use App\Http\Resources\ApprovalWorkFlowCollection;
use App\Http\Resources\ApprovalWorkFlowResource;
use App\Models\ApprovalGroup;
use App\Models\ApprovalWorkFlow;
use App\Repositories\ApprovalWorkFlowRepository;
use Exception;
use Illuminate\Database\Eloquent\ModelNotFoundException;
use Illuminate\Http\RedirectResponse;
use Illuminate\Support\Facades\DB;
use Illuminate\View\View;

class ApprovalWorkFlowController extends Controller
{
    public function __construct(
        protected ApprovalWorkFlowRepository $approvalWorkFlowRepository
    ){}

    /**
     * Display a listing of the ApprovalFlow.
     *
     */
    public function index()
    {
        $workFlows = ApprovalWorkFlow::all();
        return new ApprovalWorkFlowCollection($workFlows);
    }

    /**
     * Show the form for creating a new ApprovalFlow.
     *
     * @return View
     */
    public function create(): View
    {
        return view('approval_work_flows.create');
    }

    /**
     * Store a newly created ApprovalFlow in storage.
     *
     * @param CreateApprovalWorkFlowRequest $request
     *
     */
    public function store(CreateApprovalWorkFlowRequest $request)
    {
        /*{
            "_token": "wWjZrGhTruqtmQodkeImtskmumxE1PHFLzCAEKsU",
            "name": "flow",
            "rank": "4",
            "type": "checklist",
            "approval_group_id": [
                "1"
             ],
            "description": "flow check"
        }*/

        $input = $request->all();
        $approvalWorkFlow = new ApprovalWorkFlow();
        //begin transaction
        DB::beginTransaction();
        try {
            // $approvalWorkFlow = $this->approvalWorkFlowRepository->create($input);
            $approvalWorkFlow = ApprovalWorkFlow::create([
                'name' => $input['name'],
                'rank' => $input['rank'],
                'type' => $input['type'],
                'updated_by' => $input['updated_by'],
                'created_by' => $input['created_by'],
                'description' => $input['description'],
            ]);

            foreach (json_decode($input['approval_group_id']) as $approvalGroupId) {
                $approvalGroup = ApprovalGroup::findOrFail($approvalGroupId);
                $approvalWorkFlow->approvalGroups()->attach($approvalGroup);
            }

        } catch (ModelNotFoundException $e) {
            DB::rollback();
            throw $e;
        }
        DB::commit();

        return new ApprovalWorkFlowResource($approvalWorkFlow);
    }

    /**
     * Display the specified ApprovalFlow.
     *
     * @param int $id
     *
     * @return RedirectResponse | View
     */
    public function show(int $id): View|RedirectResponse
    {
        $approvalWorkFlow = $this->approvalWorkFlowRepository->findWithoutFail($id);

        if (empty($approvalWorkFlow)) {
            Alert::error('Approval Work Flow not found');
            return redirect(route('approvalWorkFlows.index'));
        }

        return view('approval_work_flows.show')->with('approvalWorkFlow', $approvalWorkFlow);
    }

    /**
     * Show the form for editing the specified ApprovalFlow.
     *
     * @param int $id
     *
     * @return RedirectResponse | view
     */
    public function edit(int $id): View|RedirectResponse
    {
        $approvalWorkFlow = $this->approvalWorkFlowRepository->find($id);

        if (empty($approvalWorkFlow)) {
            Alert::error('Approval Work Flow not found');

            return redirect(route('approvalWorkFlows.index'));
        }

        return view('approval_work_flows.edit', compact('approvalWorkFlow'));
    }

    /**
     * Update the specified ApprovalFlow in storage.
     *
     * @param int $id
     * @param UpdateApprovalWorkFlowRequest $request
     *
     * @return RedirectResponse
     */
    public function update(int $id, UpdateApprovalWorkFlowRequest $request): RedirectResponse
    {
        $approvalWorkFlow = $this->approvalWorkFlowRepository->find($id);

        if (empty($approvalWorkFlow)) {
            Alert::error('Approval Work Flow not found');

            return redirect(route('approvalWorkFlows.index'));
        }

        $approvalWorkFlow = $this->approvalWorkFlowRepository->update($request->all(), $id);

        Alert::toast('Approval Work Flow updated successfully.','success');

        return redirect(route('approvalWorkFlows.index'));
    }

    /**
     * Remove the specified ApprovalFlow from storage.
     *
     * @param int $id
     *
     * @return RedirectResponse
     * @throws Exception
     */
    public function destroy(int $id): RedirectResponse
    {
        $approvalWorkFlow = $this->approvalWorkFlowRepository->find($id);

        if (empty($approvalWorkFlow)) {
            Alert::error('Approval Work Flow not found');

            return redirect(route('approvalWorkFlows.index'));
        }

        $this->approvalWorkFlowRepository->delete($id);

        Alert::success('Approval Work Flow deleted successfully.');

        return redirect(route('approvalWorkFlows.index'));
    }
}
